
                   
define([
    'dojox/charting/widget/Chart',
    'dojo/dom-geometry',
    'dojo/_base/array',
    'dojo/number',
    'sna/util/I18nUtil',
    'dojo/dom-style',
    'dojo/_base/lang',
    'dojox/charting/widget/Legend',
    'dojox/charting/action2d/Magnify',
    'dojox/charting/action2d/Tooltip',
    'dojox/charting/action2d/MoveSlice',
    'dojox/lang/functional',
    'dojo/dom-construct',
    'dijit/layout/BorderContainer',
    'dijit/layout/ContentPane',
    'dojo/_base/declare',
    'dojox/charting/Chart',
    'dojox/charting/themes/Claro',
    'dojox/charting/plot2d/Scatter',
    'dojox/charting/plot2d/Lines',
    'dojox/charting/plot2d/Pie',
    'dojox/charting/plot2d/Bubble',
    'dojox/charting/plot2d/ClusteredColumns',
    'dojox/charting/plot2d/Markers',
    'dojox/charting/axis2d/Default',
    'dojox/charting/plot2d/Grid'
], function(ChartWidget, domGeometry, array, number, I18nUtil, style, lang, Legend, Magnify, Tooltip, MoveSlice, functional, domConstruct, BorderContainer, ContentPane, declare, Chart, Shrooms) {
    var getYAxis = function (subpanel) {
        var axisObject = {
            dropLabels:false,
            minorTicks:false,
            vertical:true
        };

        if (typeof subpanel.min !== 'undefined' && typeof subpanel.max !== 'undefined') {
            if (Math.abs(subpanel.max - subpanel.min) <= 1e-6) {
                axisObject.max = subpanel.max + subpanel.max / 2;
                axisObject.min = subpanel.min - subpanel.min / 2;
            }
            else if (subpanel.max < 500 && subpanel.min >= 0) {
                axisObject.max = subpanel.max;
                axisObject.min = 0;
            }
            else {
                if(subpanel.chartType === 'bar') {
                    axisObject.min = subpanel.min - 1;
                }
            }
        }

        
        axisObject.fixLower = 'major';
        axisObject.fixUpper = 'major';

        return axisObject;
    };

    return declare(null, {
        getChartRenderer:  function(subpanel, d_target, subpanelCount, chartType, options) {
            
            
            
            
            var keysToSort = Object.keys(subpanel.series);
            var sortedSeriesKeys = keysToSort.sort(function (a, b) {
                var aValue = 0;
                var bValue = 0;

                if (!a.startsWith('series')) {
                    aValue = 0;
                }
                else {
                    aValue = number.parse(a.replace('series', '').replace('_name', ''));
                }

                if (!b.startsWith('series')) {
                    bValue = 0;
                }
                else {
                    bValue = number.parse(b.replace('series', '').replace('_name', ''));
                }

                return aValue - bValue;
            });

            return function() {
                
                d_target.destroyDescendants();
                var chartOptions = lang.mixin({}, options);

                var d_container = new BorderContainer({
                    style: 'width: 100%; height: 100%;',
                    liveSplitters: false,
                    gutters: false
                });

                var formattedId = subpanel.formattedId || subpanel.chartId;
                if(subpanel.formattedId && subpanel.localizeFormattedId) {
                    formattedId = I18nUtil.get(subpanel.formattedId);
                }

                var d_titlePane = new ContentPane({
                    content: '<header>%0</header>'.tokenize(formattedId),
                    region: 'top',
                    style: 'padding: 0'
                });
                d_container.addChild(d_titlePane);
                var titleHeight = 25;

                var d_chartPane = new ContentPane({
                    style: 'width: 100%; padding: 0 !important; ',
                    region: 'center'
                });
                d_container.addChild(d_chartPane);

                
                
                var legendHeight = 10;
                if(typeof(chartOptions.disableLegend) === 'undefined' || chartOptions.disableLegend === false) {
                    legendHeight = 30;
                    var legendContent = domConstruct.toDom('<div></div>');

                    var d_legendPane = new ContentPane({
                        region: 'bottom',
                        content: legendContent,
                        style: 'padding: 0 !important;'
                    });
                    d_container.addChild(d_legendPane);
                }

                
                var columnGap = 30 - 2 * subpanel.series.category_field.length - 2 * subpanelCount;
                if(columnGap < 5) {
                    columnGap = 5;
                }

                var getStrokeColor = function() {
                    if(options && options.getStrokeColor) {
                        return options.getStrokeColor(subpanel.series.category_field, subpanel.categoryColors);
                    }

                    return undefined;
                };

                var myTheme =  Shrooms;

                
                
                if(subpanel.chartTheme && subpanel.chartTheme === 'white') {
                    myTheme.fill = myTheme.plotarea.fill = myTheme.chart.fill = "white";
                    myTheme.axis.stroke.color = myTheme.axis.tick.fontColor = 'black';
                }
                else if (subpanel.chartTheme && subpanel.chartTheme === 'blueSteel') {
                    myTheme.fill = myTheme.plotarea.fill = myTheme.chart.fill = "#454852";
                    myTheme.axis.stroke.color = myTheme.axis.tick.fontColor = 'white';
                }

                
                
                var majorTickStep = 1;
                functional.forIn(subpanel.series, function(item) {
                    if(item.length && item.length > 25) { 
                        majorTickStep = undefined;
                    }
                });

                var xAxis = {
                    minorTicks: false,
                    labelFunc: function(text, value, precision) {
                        if(precision === 0) {
                            var label = subpanel.series.category_field[value - 1];
                            var domLabel = domConstruct.toDom('<div>%0</div>'.tokenize(label));
                            label = domLabel.innerHTML;
                            
                            if(typeof(label) !== 'undefined' && !isNaN(label)) {
                                label = number.format(label);
                            }

                            if(typeof(label) !== 'undefined' && label.length > 10) {
                                label = label.substring(0, 10) + '...';
                            }

                            return label;
                        }
                    }
                };

                if(typeof(majorTickStep) !== 'undefined') {
                    xAxis.majorTickStep = majorTickStep;
                }

                
                
                var chartWidget = new ChartWidget({});
                chartWidget.placeAt(d_chartPane.domNode);

                var chart = chartWidget.chart
                    .setTheme(myTheme)
                    .addAxis('x', xAxis)
                    .addAxis('y', getYAxis(subpanel))
                    .addPlot("default", { type:chartType, markers:true, gap: columnGap, stroke: getStrokeColor()});


                
                
                
                functional.forIn(sortedSeriesKeys, function(key) {
                    var value = subpanel.series[key];

                    if(key.startsWith('series')) {
                        var seriesColor = {};
                        var seriesColorsEnabled = !(options && options.disableSeriesColors);

                        if(subpanel.seriesColors && subpanel.seriesColors[key] && seriesColorsEnabled) {
                            seriesColor = {
                                stroke: subpanel.seriesColors[key].replace('0x', '#'),
                                fill: subpanel.seriesColors[key].replace('0x', '#')
                            };
                        }

                        if(options && options.seriesModifier) {
                            value = options.seriesModifier(value, subpanel.series.category_field, subpanel.categoryColors);
                        }

                        if(subpanel.seriesLabels && subpanel.seriesLabels[key]) {
                            chart.addSeries(subpanel.seriesLabels[key], value, seriesColor);
                        }
                        else {
                            chart.addSeries(key, value, seriesColor);
                        }
                    }
                });

                
                var tip = new Tooltip(chart,"default", {
                    text: function(element) {
                        
                        if(chartType === 'Scatter' || chartType === 'Lines') {
                            var finalElement = "";
                            array.forEach(element.chart.series, function(series, index) {
                                var d = series.data[element.index];
                                if(typeof(d) !== 'undefined' && (d === element.y || (d && d.y && d.y === element.y))) {
                                    var formattedValue = element.y;
                                    if(!isNaN(formattedValue)) {
                                        formattedValue = number.format(formattedValue);
                                    }

                                    var formattedLabel = subpanel.series.category_field[element.index];
                                    if(!isNaN(formattedLabel)) {
                                        formattedLabel = number.format(formattedLabel);
                                    }

                                    finalElement += '<div>%0</div><div style="padding-bottom:5px;"><b>%1:</b> %2</div>'.tokenize(formattedLabel, series.name, formattedValue);
                                }
                            });
                        }
                        else {
                            var formattedValue = element.y;
                            if(!isNaN(formattedValue)) {
                                formattedValue = number.format(formattedValue);
                            }

                            var formattedLabel = subpanel.series.category_field[element.index];
                            if(!isNaN(formattedLabel)) {
                                formattedLabel = number.format(formattedLabel);
                            }

                            return '<div>%0</div><div><b>%1:</b> %2</div>'.tokenize(formattedLabel, element.run.name, formattedValue);
                        }

                        return finalElement;
                    }
                });

                
                chart.render();
                d_target.addChild(d_container);
                d_target.startup();

                if(typeof(chartOptions.disableLegend) === 'undefined' || chartOptions.disableLegend === false) {
                    var legend = new Legend({chart: chart, horizontal: true}, legendContent);
                    legend.startup();
                    legendHeight = domGeometry.position(legend.domNode).h;
                }

                if(chartOptions.pluginsCreator) {
                    chartOptions.pluginsCreator(chart);
                }
                var themeOffset = 10;
                style.set(d_chartPane.domNode, 'overflow', 'hidden');
                chart.resize(style.get(d_chartPane.domNode, 'width'), style.get(d_target.domNode, 'height') - legendHeight - titleHeight - themeOffset);
                d_target.resize();
            };
        },

        getPieRenderer:function (subpanel, d_target, subpanelCount) {
            return this.getChartRenderer(subpanel, d_target, subpanelCount, 'Pie', {
                disableSeriesColors: true,
                disableLegend:true,
                pluginsCreator:function (chart) {
                    new MoveSlice(chart, "default");
                },
                
                seriesModifier: function(series, categoryField, categoryColors) {
                    if(!(categoryColors && Object.keys(categoryColors).length > 0)) {
                        return series;
                    }

                    var newSeries = [];

                    functional.forIn(series, function(value, index) {
                        
                        
                        
                        if(categoryField[index] && categoryColors[categoryField[index]]) {
                            newSeries[index] = {
                                y: value,
                                color: categoryColors[categoryField[index]].replace('0x', '#')
                            };
                        }
                        else {
                            
                            
                            newSeries[index] = {
                                y:value,
                                color: Shrooms.next().marker
                            };
                        }
                    });

                    return newSeries;
                },
                getStrokeColor: function(categoryField, categoryColors) {
                    
                    if(categoryColors && Object.keys(categoryColors).length > 0 && categoryField && categoryField.length === Object.keys(categoryColors).length) {
                        return '#000000';
                    }
                    else {
                        return '#ffffff';
                    }
                }
            });
        }
    });
});
               